#
# Copyright (c) 2015 Midokura SARL, All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import subprocess
import tempfile

from charmhelpers.core.services import base
import yaml

JUJU_HIERA_PATH = '/var/lib/hiera/'


class InstallCallback(base.ManagerCallback):
    def __call__(self, manager, service_name, event_name):
        service = manager.get_service(service_name)
        context = {}
        for ctx in service.get('required_data', []):
            context.update(ctx)

        with tempfile.NamedTemporaryFile(mode='w+t') as tmp:
            if self._fingerprint_key in context:
                tmp.write("apt::key { 'key': \n    key => '%s',\n" %
                          context[self._fingerprint_key])
                if self._content_key in context:
                    tmp.write("    key_content => '%s',\n" %
                              context[self._content_key])
                tmp.write("} ->\n")
            tmp.write("class { '%s':\n}\n" % self._component)
            tmp.flush()
            _apply(tmp.name)

    def __init__(self, component):
        self._component = component
        self._fingerprint_key = 'midonet_key_fingerprint'
        self._content_key = 'midonet_key_content'


def _apply(path, debug=False):
    """Apply a puppet manifest."""
    cmd = ['puppet', 'apply']
    if debug:
        cmd.append('-d')
    cmd.append(path)
    subprocess.check_call(cmd)


apply = _apply


def module_install(module_name, force=False, debug=False):
    """Install or upgrade a puppet module."""
    cmd = ['puppet', 'module']
    if module_name in subprocess.check_output(['puppet', 'module', 'list']):
        cmd.append('upgrade')
    else:
        cmd.append('install')
    if force:
        cmd.append('-f')
    if debug:
        cmd.append('-d')
    cmd.append(module_name)
    subprocess.check_call(cmd)


def set_juju_hierarchy(attempts=10, extra_data_dirs=None):
    """Sets a puppet datadir lookup to find our customizations.

    Extra data dirs must be relative to /var/lib/hiera
    """
    if attempts < 1:
        raise RuntimeError('Failed to set juju puppet hierarchy')

    hiera_path = '/etc/puppet/hiera.yaml'
    try:
        with open(hiera_path) as hiera:
            hiera_info = yaml.load(hiera)
    except IOError as ioe:
        if ioe.errno != os.errno.ENOENT:
            raise
        hiera_info = None

    if hiera_info is None:
        hiera_info = {
            ':backends': ['yaml', 'module_data'],
            ':logger': 'console',
            ':hierarchy': ['common'],
            ':yaml': {
                ':datadir': JUJU_HIERA_PATH}}

    if extra_data_dirs is not None:
        if ':hierarchy' not in hiera_info:
            hiera_info[':hierarchy'] = ['common']
        _ensure_dirs(extra_data_dirs)
        for dir_path in extra_data_dirs:
            dir_path_common = os.path.join(dir_path, 'common')
            if dir_path_common not in hiera_info[':hierarchy']:
                hiera_info[':hierarchy'].insert(0, dir_path_common)

    hiera_info[':yaml'][':datadir'] = JUJU_HIERA_PATH

    with open(hiera_path, 'w') as hiera:
        yaml.dump(hiera_info, stream=hiera, default_flow_style=False,
                  explicit_start=True, indent=2)


def _ensure_dirs(dirs):
    """Creates the directories in dirs if they are not present."""
    for dir_path in dirs:
        try:
            os.makedirs(dir_path)
        except OSError as ose:
            if ose.errno != os.errno.EEXIST:
                raise
