#!/usr/bin/env python3
# vim: tabstop=4 shiftwidth=4 softtabstop=4 filetype=python
#
# Copyright (c) 2015 Midokura Europe SARL, All Rights Reserved.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import amulet

from charmhelpers.contrib.openstack.amulet.utils import (
    OpenStackAmuletUtils,
    DEBUG,  # flake8: noqa
    ERROR
)

# Use DEBUG to turn on debug logging
u = OpenStackAmuletUtils(DEBUG)

SETUP_TIMEOUT = 9600


class MidonetBasicDeployment():
    def __init__(self, ubuntu_series, openstack=None, midonet_release=None):
        self.os_release = openstack
        self.series = ubuntu_series
        self.midonet_release = midonet_release
        self.d = amulet.Deployment(series=self.series)
        self._add_services()
        self._add_relations()
        self._configure_services()
        self._deploy()
        self._initialize_tests()

    def _add_services(self):
        self.d.add('mysql', charm='cs:trusty/mysql', units=1,
                   series=self.series)
        self.d.add('rabbitmq-server', charm='cs:trusty/rabbitmq-server',
                   units=1, series=self.series)
        self.d.add('cassandra', charm='cassandra', units=1,
                   branch='lp:charms/cassandra', series=self.series)
        self.d.add('keystone', charm='keystone', units=1, series=self.series,
                    branch='lp:~celebdor/charms/trusty/keystone/trunk')
        self.d.add('nova-compute', charm='nova-compute', units=1,
                   branch='lp:~celebdor/charms/trusty/nova-compute/trunk',
                   series=self.series)
        self.d.add('neutron-api', charm='neutron-api', units=1,
                   branch='lp:~celebdor/charms/trusty/neutron-api/trunk',
                   series=self.series)
        self.d.add('nova-cloud-controller', charm='nova-cloud-controller',
               branch='lp:~celebdor/charms/trusty/nova-cloud-controller/trunk',
               units=1, series=self.series)
        self.d.add('zookeeper', charm='cs:trusty/zookeeper', units=1,
                   series=self.series)
        self.d.add('midonet-api', charm='midonet-api', units=1,
                   branch='lp:~celebdor/charms/trusty/midonet-api/trunk',
                   series=self.series)
        self.d.add('midonet-agent', charm='midonet-agent', units=1,
                   branch='lp:~celebdor/charms/trusty/midonet-agent/trunk',
                   series=self.series)
        self.d.add('neutron-agents-midonet', charm='neutron-agents-midonet',
              branch='lp:~celebdor/charms/trusty/neutron-agents-midonet/trunk',
              series='trusty')

    def _add_relations(self):
        self.d.relate('mysql:shared-db', 'keystone:shared-db')
        self.d.relate('mysql:shared-db', 'nova-cloud-controller:shared-db')
        self.d.relate('rabbitmq-server:amqp', 'nova-compute:amqp')
        self.d.relate('rabbitmq-server:amqp', 'nova-cloud-controller:amqp')
        self.d.relate('rabbitmq-server:amqp', 'neutron-api:amqp')
        self.d.relate('nova-cloud-controller:identity-service',
                      'keystone:identity-service')

        self.d.relate('neutron-api:identity-service',
                      'keystone:identity-service')
        self.d.relate('nova-compute:cloud-compute',
                      'nova-cloud-controller:cloud-compute')
        self.d.relate('neutron-api:neutron-api',
                      'nova-cloud-controller:neutron-api')

        self.d.relate('keystone:identity-service', 'midonet-api:keystone')
        self.d.relate('zookeeper:zookeeper', 'midonet-api:zookeeper')
        self.d.relate('midonet-agent:host', 'midonet-api:host')
        self.d.relate('neutron-api:midonet', 'midonet-api:midonet-api')

        self.d.relate('neutron-agents-midonet:neutron_agents',
                      'nova-cloud-controller:quantum-network-service')

        self.d.relate('neutron-agents-midonet:neutron-plugin-api',
                      'neutron-api:neutron-plugin-api')

        self.d.relate('midonet-agent:neutron-plugin',
                      'nova-compute:neutron-plugin')

        self.d.relate('midonet-agent:host', 'neutron-api:midonet-host')
        self.d.relate('midonet-agent:cassandra', 'cassandra:database')
        self.d.relate('midonet-agent:zookeeper', 'zookeeper:zookeeper')

    def _configure_services(self):
        self.d.configure('keystone', {
                         'enable-pki': 'false',
                         'openstack-origin': self.os_release})

        self.d.configure('cassandra', {
            'allow-single-node': True,
            'cluster-name': 'midonet',
            'apt-repo-key': '7E41C00F85BFC1706C4FFFB3350200F2B999A372',
            'apt-repo-spec':
            'deb http://debian.datastax.com/community 2.0 main',
            'extra_packages': 'openjdk-7-jre-headless dsc20'})

        self.d.configure('mysql', {'max-connections': 2000})

        self.d.configure('nova-compute', {
                         'openstack-origin': self.os_release,
                         'virt-type': 'qemu',
                         'flat-interface': 'eth0',
                         'manage-neutron-plugin-legacy-mode': 'false'})

        self.d.configure('nova-cloud-controller', {
                         'openstack-origin': self.os_release,
                         'network-manager': 'Neutron',
                         'shared_secret': 'secret'})

        self.d.configure('neutron-api', {
                         'neutron-plugin': 'midonet',
                         'neutron-security-groups': 'True',
                         'neutron-external-network': 'Public_Network',
                         'l2-population': 'False',
                         'openstack-origin': self.os_release,
                         'midonet-release': self.midonet_release})

        self.d.configure('midonet-api',
                         {'midonet-release': self.midonet_release})

        self.d.configure('neutron-agents-midonet',
                         {'shared_secret': 'secret'})

        self.d.configure('midonet-agent',
                         {'midonet-release': self.midonet_release})

    def _deploy(self):
        try:
            self.d.setup(timeout=SETUP_TIMEOUT)
            self.d.sentry.wait(timeout=SETUP_TIMEOUT)
        except amulet.helpers.TimeoutError:
            amulet.raise_status(amulet.SKIP,
                                msg="Environment wasn't stood up in time")

    def _initialize_tests(self):
        self.zookeeper_sentry = self.d.sentry.unit['zookeeper/0']
        self.keystone_sentry = self.d.sentry.unit['keystone/0']
        self.cassandra_sentry = self.d.sentry.unit['cassandra/0']
        self.nova_compute_sentry = self.d.sentry.unit['nova-compute/0']
        self.mn_api_sentry = self.d.sentry.unit['midonet-api/0']
        self.mn_agent_sentry = self.d.sentry.unit['midonet-agent/0']
        self.agents_mn_sentry = self.d.sentry.unit['neutron-agents-midonet/0']

        self.keystone_api_relation = self.keystone_sentry.relation(
             'identity-service', 'midonet-api:keystone')
        self.zookeeper_api_relation = self.zookeeper_sentry.relation(
             'zookeeper', 'midonet-api:zookeeper')
        self.zk_host_relation = self.zookeeper_sentry.relation(
                                'zookeeper', 'midonet-agent:zookeeper')
        self.cs_host_relation = self.cassandra_sentry.relation(
            'database', 'midonet-agent:cassandra')
        self.api_host_relation = self.mn_api_sentry.relation(
            'host', 'midonet-agent:host')

    def run_tests(self):
        for test in dir(self):
            if test.startswith('test_'):
                getattr(self, test)()

    def test_mn_agent(self):
        u.log.debug('Checking if the key has been correctly fetched...')
        apt_key_out, apt_key_exit = self.mn_agent_sentry.run(
                                    'apt-key list | grep Midokura')
        if 'Midokura' not in apt_key_out:
            message = ("error, Midokura repository key is not installed")
            amulet.raise_status(amulet.FAIL, msg=message)

        u.log.debug('Checking if midolman package has been installed...')
        dpkg_midol_out, dpkg_exit = self.mn_agent_sentry.run(
                                    'dpkg -l | grep midolman')
        if 'midolman' not in dpkg_midol_out:
            message = ("error, midolman package is not installed")
            amulet.raise_status(amulet.FAIL, msg=message)

        u.log.debug('Checking zookeeper state...')
        zk_out, zk_exit = self.zookeeper_sentry.run(
                               'echo ruok | nc 127.0.0.1 2181')
        if 'imok' not in zk_out:
            message = ("zookeeper in error state: %s" % zk_exit)
            amulet.raise_status(amulet.FAIL, msg=message)

        u.log.debug('Checking if cassandra is running in a non-error state...')
        cs_out, _ = self.cassandra_sentry.run(
           'nodetool -host 127.0.0.1 status')
        if 'Up' not in cs_out:
            message = ("error: cassandra in error state: %s" % cs_out)
            amulet.raise_status(amulet.FAIL, msg=message)

        u.log.debug('Checking if the host has been successfully added...')
        cli_out, _ = self.mn_api_sentry.run('midonet-cli -e host list')
        if 'host' not in cli_out:
            message = ("error, midonet host not in tunnel-zone")
            amulet.raise_status(amulet.FAIL, msg=message)
